import { world, system } from "@minecraft/server";
import { msToTime } from "../../Assets/Util";
import lang from "../../Data/Languages/lang";
import { triggerEvent } from "./eventHandler";
function checksBan(player) {
    const info = player.getDynamicProperty("isBanned");
    const baninfo = info === undefined ? undefined : JSON.parse(info);
    const unbanListingString = world.getDynamicProperty("unbanListing");
    const unbanListing = unbanListingString ? JSON.parse(unbanListingString) : [];
    if (unbanListing.includes(player.name)) {
        world.setDynamicProperty("unbanListing", JSON.stringify(unbanListing.filter(name => name !== player.name)));
        player.setDynamicProperty("isBanned", undefined);
        return;
    }
    if (baninfo === undefined)
        return;
    let reason;
    let by;
    let time;
    try {
        reason = baninfo.reason;
        by = baninfo.by;
        time = baninfo.time;
    }
    catch {
        console.log("baninfo is not a BanInfo object, unbanned");
        player.setDynamicProperty("isBanned", undefined);
    }
    if (time !== "forever") {
        if (Date.now() > time) {
            player.setDynamicProperty("isBanned", undefined);
            return;
        }
    }
    const timeLeft = time === "forever" ? "forever" : msToTime(time - Date.now());
    let timeTherShold;
    if (timeLeft === "forever") {
        timeTherShold = "forever";
    }
    else {
        const { days: d, hours: h, minutes: m, seconds: s } = timeLeft;
        timeTherShold = `${d} days, ${h} hours, ${m} minutes, ${s} seconds`;
    }
    try {
        player.runCommand(`kick "${player.name}" §r\n§c§l${lang(".banHandler.banned")}${lang(".banHandler.format").replace("%a", timeTherShold).replace("%b", reason).replace("%c", by)}`);
    }
    catch {
        triggerEvent(player, "matrix:kick");
    }
}
function ban(player, reason, by, time) {
    system.run(() => {
        player.setDynamicProperty("isBanned", JSON.stringify({
            isBanned: true,
            reason: reason,
            by: by,
            time: time
        }));
        checksBan(player);
    });
}
function unban(playerName) {
    const unbanListing = JSON.parse(world.getDynamicProperty("unbanListing"));
    system.run(() => world.setDynamicProperty("unbanListing", JSON.stringify([...unbanListing, playerName])));
}
function unbanRemove(playerName) {
    const unbanListing = JSON.parse(world.getDynamicProperty("unbanListing"));
    if (!unbanListing.includes(playerName))
        return false;
    system.run(() => world.setDynamicProperty("unbanListing", JSON.stringify(unbanListing.filter(name => name !== playerName))));
    return true;
}
function unbanList() {
    return JSON.parse(world.getDynamicProperty("unbanListing") ?? "[]");
}
world.afterEvents.playerSpawn.subscribe(({ player, initialSpawn }) => {
    if (!initialSpawn)
        return;
    checksBan(player);
});
export { ban, unban, unbanRemove, unbanList };
