import { world, system } from "@minecraft/server";
import config from "../../Data/Config.js";
import { isAdmin, kick } from "../../Assets/Util.js";
import lang from "../../Data/Languages/lang.js";
export { antiSpamModule };
;
const previousMessage = new Map();
const spamData = new Map();
async function spammingWarner(player, data) {
    data.warnings++;
    if (data.warnings <= config.antiSpam.kickThreshold) {
        player.sendMessage(`§bMatrix §7>§g ${lang(".Spam.slowdown")} §8(${data.warnings}/${config.antiSpam.kickThreshold})`);
    }
    system.runTimeout(() => {
        data.warnings = 0;
        spamData.set(player.id, data);
    }, config.antiSpam.timeout);
    if (data.warnings > config.antiSpam.kickThreshold) {
        system.run(() => kick(player, 'spamming', 'Matrix AntiCheat'));
        world.sendMessage(`§bMatrix §7>§g ${lang(".Spam.kicked").replace("%a", player.name)}`);
    }
}
;
function antiSpamModule(message, player) {
    const toggle = (world.getDynamicProperty("antiSpam") ?? config.antiSpam.enabled);
    if (toggle !== true || isAdmin(player))
        return false;
    let isSpamming = false;
    if (previousMessage.has(player.id) && previousMessage.get(player.id) === message) {
        system.run(() => player.sendMessage('§bMatrix §7>§g ' + lang(".Spam.repeated")));
        isSpamming = true;
    }
    else {
        previousMessage.set(player.id, message);
    }
    if (message.length > config.antiSpam.maxCharacterLimit) {
        player.sendMessage(`§bMatrix §7>§g ${lang(".Spam.long")} §8(${message.length}/${config.antiSpam.maxCharacterLimit})`);
        isSpamming = true;
    }
    else if (config.chatFilter.some((word) => message.toLowerCase().includes(word))) {
        system.run(() => player.sendMessage(`§bMatrix §7>§g ${lang(".Spam.filter")}`));
        isSpamming = true;
    }
    // Check if the message contain a blacklisted word
    if (config.blacklistedMessages.some((word) => message.toLowerCase().includes(word))) {
        // cancel the message
        isSpamming = true;
        // increase their warning
        let warningTime = player.blacklistMsgWarn ?? 0;
        warningTime++;
        player.blacklistMsgWarn = warningTime;
        // if warning time is smaller than 2, send a warning message else kick them
        if (warningTime < 5) {
            system.run(() => player.sendMessage(`§bMatrix §7>§g ${lang(".Spam.blacklist")} §8(${warningTime}/4)`));
            isSpamming = true;
        }
        system.run(() => {
            kick(player, 'blacklisted message', 'Matrix AntiCheat');
            world.sendMessage(`§bMatrix §7>§g ${lang(".Spam.kickedBlacklist").replace("%a", player.name)}`);
        });
        isSpamming = true;
    }
    else {
        player.blacklistMsgWarn = 0;
    }
    const data = spamData.get(player.id) || {
        lastMessageTimes: [],
        warnings: 0
    };
    //log the message time
    const currentTime = Date.now();
    data.lastMessageTimes.push(currentTime);
    //remove the message time if it's older than 1 second
    if (data.lastMessageTimes.length > config.antiSpam.maxMessagesPerSecond) {
        data.lastMessageTimes.shift();
    }
    //if the player send too many messages in 1 second, flag them
    if (data.lastMessageTimes.length >= config.antiSpam.maxMessagesPerSecond &&
        data.lastMessageTimes[data.lastMessageTimes.length - 1] - data.lastMessageTimes[0] < config.antiSpam.timer) {
        system.run(() => spammingWarner(player, data));
        isSpamming = true;
    }
    //set the new spammer data
    spamData.set(player.id, data);
    return isSpamming;
}
;
world.afterEvents.playerLeave.subscribe(({ playerId }) => {
    spamData.delete(playerId);
    previousMessage.delete(playerId);
});
