import { system, world } from "@minecraft/server";
import { MinecraftBlockTypes, MinecraftEffectTypes } from "../../node_modules/@minecraft/vanilla-data/lib/index";
import { flag, isAdmin, c } from "../../Assets/Util";
import lang from "../../Data/Languages/lang";
function getSpeedIncrease(speedEffect) {
    if (speedEffect === undefined)
        return 0;
    return (speedEffect?.amplifier + 1) * 0.0476;
}
const lastPosition = new Map();
/**
 * @author RaMiGamerDev
 * @description A strong check for no slow, it detect no slow in a high accuracy
 */
async function AntiNoSlow(player) {
    const config = c();
    //get the player location
    const playerLocation = player.location;
    //get the last position
    const playerLastPos = lastPosition.get(player.id) ?? player.location;
    //get the velocity
    const { x: velocityX, z: velocityZ } = player.getVelocity();
    //get the no slow buffer
    const buffer = (player.noSlowBuffer ?? 0);
    //check if the player's is in the Web
    const headWeb = player.dimension.getBlock({
        x: Math.floor(player.location.x),
        y: Math.floor(player.location.y) + 1,
        z: Math.floor(player.location.z)
    })?.typeId === MinecraftBlockTypes.Web;
    const bodyWeb = player.dimension.getBlock({
        x: Math.floor(player.location.x),
        y: Math.floor(player.location.y),
        z: Math.floor(player.location.z)
    })?.typeId === MinecraftBlockTypes.Web;
    //if the player isn't in the web, set the last position
    if (!headWeb && !bodyWeb) {
        lastPosition.set(player.id, playerLocation);
    }
    //get the player speed
    const playerSpeed = Math.hypot(velocityX, velocityZ);
    //get the limit increase from the speed effect
    const limitIncrease = getSpeedIncrease(player.getEffect(MinecraftEffectTypes.Speed));
    //check if the player is in the web and the player speed is lower than the max speed
    if (headWeb === true || bodyWeb === true) {
        if (playerSpeed <= (0.09 + limitIncrease)) {
            lastPosition.set(player.id, playerLocation);
            player.noSlowBuffer = 0;
        }
        else {
            //flag the player, if the buffer is higher than the max buffer, teleport the player back
            player.noSlowBuffer++;
            if (buffer + 1 > config.antiNoSlow.maxNoSlowBuff) {
                player.noSlowBuffer = 0;
                //A - false positive: very low, efficiency: high
                flag(player, "NoSlow", "A", config.antiNoSlow.maxVL, config.antiNoSlow.punishment, [`${lang(">playerSpeed")}:${playerSpeed.toFixed(2)}`]);
                if (!config.slient)
                    player.teleport(playerLastPos);
            }
        }
    }
    else {
        if (buffer > 0) {
            player.noSlowBuffer = 0;
            lastPosition.set(player.id, playerLocation);
        }
    }
    //check if player speed while using item is too high
    if (!player.getEffect(MinecraftEffectTypes.Speed) && player.lastItemUsed && Date.now() - player.lastItemUsed >= config.antiNoSlow.itemUseTime && playerSpeed > config.antiNoSlow.maxUsingItemTherehold && player.isOnGround) {
        const isIceBelow = player.dimension.getBlock({
            x: Math.floor(player.location.x),
            y: Math.floor(player.location.y) - 1,
            z: Math.floor(player.location.z)
        })?.typeId === MinecraftBlockTypes.Ice;
        if (!isIceBelow) {
            player.teleport(player.location);
            player.addTag("matrix:item-disabled");
            system.runTimeout(() => player.removeTag("matrix:item-disabled"), config.antiNoSlow.timeout);
            //B- false positive: low, efficiency: mid
            flag(player, "NoSlow", "B", config.antiNoSlow.maxVL, config.antiNoSlow.punishment, [`${lang(">playerSpeed")}:${playerSpeed.toFixed(2)}`]);
        }
    }
}
const antiNoSlow = () => {
    const players = world.getAllPlayers();
    for (const player of players) {
        if (isAdmin(player))
            continue;
        AntiNoSlow(player);
    }
};
let id;
export default {
    enable() {
        id = system.runInterval(antiNoSlow);
    },
    disable() {
        lastPosition.clear();
        system.clearRun(id);
    }
};
